//========================================================================================
//
//	File:				Content.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "Form.hpp"

#ifndef CONTENT_H
#include "Content.h"
#endif

#ifndef PART_H
#include "Part.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment Form
#endif

FW_DEFINE_AUTO(CFormContent)

//========================================================================================
//	CFormContent class
//========================================================================================

//----------------------------------------------------------------------------------------
// CFormContent constructor
//----------------------------------------------------------------------------------------

CFormContent::CFormContent(Environment* ev, CFormPart* part) :
	FW_CContent(ev, part),
	fTextData(""),
	fCentered(FALSE),
	fFormPart(part)
{
	// ----- Initialize the text data strings -----
	FW_CSharedLibraryResourceFile resFile(ev);

	FW_CString32 platformString;
	::FW_LoadStringByID(ev, resFile, kFormPartStrings, MULTISTRINGRES, kFirstString, fTextData);
	::FW_LoadStringByID(ev, resFile, kFormPartStrings, MULTISTRINGRES, kPlatformString, platformString);
	fTextData += platformString;
}

//----------------------------------------------------------------------------------------
// CFormContent destructor
//----------------------------------------------------------------------------------------

CFormContent::~CFormContent()
{
}

//----------------------------------------------------------------------------------------
// CFormContent::GetTextData
//----------------------------------------------------------------------------------------

const FW_CString& CFormContent::GetTextData()
{
	return fTextData;
}

//----------------------------------------------------------------------------------------
// CFormContent::SetTextData
//----------------------------------------------------------------------------------------

void CFormContent::SetTextData(Environment* ev, const FW_CString& newText)
{
	fTextData = newText;
	fFormPart->PartChanged(ev);
}

//----------------------------------------------------------------------------------------
// CFormContent::ClearTextData
//----------------------------------------------------------------------------------------

void CFormContent::ClearTextData(Environment* ev)
{
/*	FW_CString32 newString;	[MEB] Jim - reveals bug in Strings when trying to ReplaceAll with a string that's too big */
	FW_CString newString;
	FW_CSharedLibraryResourceFile resFile(ev);
	::FW_LoadStringByID(ev, resFile, kFormPartStrings, MULTISTRINGRES, kBlankString, newString);

	fTextData = newString;
	fFormPart->PartChanged(ev);					// Notify the proper authorities
}

//----------------------------------------------------------------------------------------
// CFormContent::Externalize
//----------------------------------------------------------------------------------------

void CFormContent::Externalize(Environment* ev,
								 ODStorageUnit* storageUnit,
								 FW_EStorageKinds storageKind,
								 FW_CCloneInfo* cloneInfo)
{
	FW_UNUSED(cloneInfo);

	ExternalizePartKind(ev, storageUnit);

	if (storageKind != FW_kPartStorage)
		ExternalizeText(ev, storageUnit);
}

//----------------------------------------------------------------------------------------
// CFormContent::ExternalizePartKind
//----------------------------------------------------------------------------------------

void CFormContent::ExternalizePartKind(Environment* ev, ODStorageUnit* storageUnit)
{
	// Write the two text data strings to storage
	FW_PStorageUnitSink suSink(ev, storageUnit, kODPropContents, fFormPart->GetPartKind(ev));
	FW_CWritableStream archive(suSink);
	unsigned long stringCount = 1;

	archive << stringCount;
	archive << fCentered;
	archive << fTextData;
}

//----------------------------------------------------------------------------------------
// CFormContent::ExternalizeText
//----------------------------------------------------------------------------------------

void CFormContent::ExternalizeText(Environment* ev, ODStorageUnit* destinationSU)
{
#ifdef FW_BUILD_MAC
	// Also write the string in text format
	FW_SUAddPropValue(ev, destinationSU, kODPropContents, FW_CPart::gMacTEXTDataType);
	FW_PStorageUnitSink suSink(ev, destinationSU, kODPropContents, FW_CPart::gMacTEXTDataType);
	FW_CWritableStream stream(suSink);
	stream.Write(fTextData.RevealBuffer(), fTextData.GetByteLength());
#endif
}

//----------------------------------------------------------------------------------------
// CFormContent::Internalize
//----------------------------------------------------------------------------------------

FW_Boolean CFormContent::Internalize(Environment* ev,
									 ODStorageUnit* sourceSU, 
									 FW_EStorageKinds storageKind,
									 FW_CCloneInfo* cloneInfo)
{
	FW_UNUSED(cloneInfo);
	FW_Boolean internalized = FALSE;

	if (FW_SUExistsThenFocus(ev, sourceSU, kODPropContents, fFormPart->GetPartKind(ev)))
		internalized = this->InternalizePartKind(ev, sourceSU);
#ifdef FW_BUILD_MAC
	else if (FW_SUExistsThenFocus(ev, sourceSU, kODPropContents, FW_CPart::gMacTEXTDataType))	// 'TEXT'
		internalized = this->InternalizeText(ev, sourceSU);
#endif

	if (internalized && storageKind != FW_kPartStorage)
		fFormPart->PartChanged(ev);

	return internalized;
}

//----------------------------------------------------------------------------------------
//	CFormContent::InternalizeText
//----------------------------------------------------------------------------------------

FW_Boolean CFormContent::InternalizeText(Environment* ev, ODStorageUnit* storageUnit)
{
	FW_Boolean result = FALSE;

#ifdef FW_BUILD_MAC
	unsigned long textSize = storageUnit->GetSize(ev);
	if (textSize > 0)
	{
		char buffer[256];
		if (textSize > 255) 
			textSize = 255;
		FW_CByteArray byteArray;
		storageUnit->GetValue(ev, textSize, byteArray);
		byteArray.CopyBuffer(&buffer, textSize);

		fTextData = "";
		fTextData.Append((const FW_Char*) &buffer, textSize);

		result = TRUE;
	}
#endif

	return result;
}

//----------------------------------------------------------------------------------------
//	CFormContent::InternalizePartKind
//----------------------------------------------------------------------------------------

FW_Boolean CFormContent::InternalizePartKind(Environment* ev, ODStorageUnit* storageUnit)
{
	FW_Boolean internalized = TRUE;

	if (storageUnit->GetSize(ev) != 0)
	{
		// Read the text data strings from storage
		FW_PStorageUnitSink suSink(ev, storageUnit, kODPropContents, fFormPart->GetPartKind(ev));
		FW_PBufferedSink sink(ev, suSink);
		FW_CReadableStream archive(sink);

		unsigned long stringCount;
		archive >> stringCount;
		archive >> fCentered;
		if (stringCount > 0)
			archive >> fTextData;
	}

	return internalized;
}

//----------------------------------------------------------------------------------------
// CFormContent::CenterText
//----------------------------------------------------------------------------------------

void CFormContent::CenterText(Environment* ev, FW_Boolean state)
{
	fCentered = state;
	fFormPart->PartChanged(ev);
}

