//========================================================================================
//
//	File:				Frame.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "Form.hpp"

#ifndef FRAME_H
#include "Frame.h"
#endif

#ifndef PART_H
#include "Part.h"
#endif

#ifndef COMMANDS_H
#include "Commands.h"
#endif

#ifndef CONTENT_H
#include "Content.h"
#endif

// ----- gadgets -----

#ifndef PICTVIEW_H
#include "PictView.h"
#endif

#ifndef FWEDVIEW_H
#include "FWEdView.h"
#endif

#ifndef FWEDVIEW_H
#include "FWEdView.h"
#endif

#ifndef FWEDVIEW_H
#include "FWEdView.h"
#endif

#ifndef FWEDVIEW_H
#include "FWEdView.h"
#endif

#ifndef FWEDVIEW_H
#include "FWEdView.h"
#endif

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

#ifndef FWCLUSTR_H
#include "FWClustr.h"
#endif

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

#ifndef FWLISTBX_H
#include "FWListBx.h"
#endif

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

#ifndef FWPOPUP_H
#include "FWPopup.h"
#endif

#ifndef FWPOPUP_H
#include "FWPopup.h"
#endif

#ifndef FWCLUSTR_H
#include "FWClustr.h"
#endif

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

#ifndef FWCLUSTR_H
#include "FWClustr.h"
#endif

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

// ----- Framework Layer -----

#ifndef FWCONTXT_H
#include "FWContxt.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

#ifndef FWGROWBX_H
#include "FWGrowBx.h"
#endif

#ifndef FWSCLBAR_H
#include "FWSclBar.h"
#endif

// ----- OS Layer -----

#ifndef FWMENU_H
#include "FWMenu.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWALERT_H
#include "FWAlert.h"
#endif

// ----- Graphic Includes -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWTXTBOX_H
#include "FWTxtBox.h"
#endif

#ifndef FWRECSHP_H
#include "FWRecShp.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_ODDragItemIterator_xh
#include <DgItmIt.xh>
#endif

//========================================================================================
// Runtime Information
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment Form
#endif

FW_DEFINE_AUTO(CFormFrame)
FW_DEFINE_CLASS_M2(CFormFrame, FW_CFrame, FW_MReceiver)

//----------------------------------------------------------------------------------------
// CFormFrame constructor
//----------------------------------------------------------------------------------------
CFormFrame::CFormFrame(Environment* ev, ODFrame* odFrame, FW_CPresentation* presentation, CFormContent* content)
	: FW_CFrame(ev, odFrame, presentation, content->GetPart(ev)),
	FW_MDraggableFrame(ev, this),
	FW_MDroppableFrame(ev, this),	
	fFormContent(content)
{
}

//----------------------------------------------------------------------------------------
// CFormFrame destructor
//----------------------------------------------------------------------------------------

CFormFrame::~CFormFrame()
{
}

//----------------------------------------------------------------------------------------
//	CFormFrame::CanAcceptDrop
//----------------------------------------------------------------------------------------

ODDragResult CFormFrame::CanAcceptDrop(Environment* ev, ODDragItemIterator* dragInfo)
{
	ODDragResult acceptDrop = FW_MDroppableFrame::CanAcceptDrop(ev, dragInfo);

	// ----- Test for 'TEXT' also -----
#ifdef FW_BUILD_MAC
	if (!acceptDrop)
	{
		for (ODStorageUnit* dragSU = dragInfo->First(ev); dragSU; dragSU = dragInfo->Next(ev))
			if (dragSU->Exists(ev, kODPropContents, FW_CPart::gMacTEXTDataType, 0))	// 'TEXT' on Scrap
				return TRUE;
	}
#endif

	return acceptDrop;
}

//----------------------------------------------------------------------------------------
//	CFormFrame::DoMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean CFormFrame::DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	FW_Boolean result = FALSE;

	if (GetPresentation(ev)->GetSelection(ev)->IsMouseInDraggableItem(ev, this, theMouseEvent, FALSE))
	{
		result = this->Drag(ev, theMouseEvent);
	}

	return result;
}

//----------------------------------------------------------------------------------------
// CFormFrame::DoMenu
//----------------------------------------------------------------------------------------

FW_Boolean CFormFrame::DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent)
{
	FW_Boolean menuHandled = TRUE;
	ODCommandID id = theMenuEvent.GetCommandID(ev);

	switch (id)
	{
		default:
			menuHandled = FALSE;
	}

	return menuHandled;
}

//----------------------------------------------------------------------------------------
// CFormFrame::DoAdjustMenus
//----------------------------------------------------------------------------------------

FW_Boolean CFormFrame::DoAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, 
									  FW_Boolean hasMenuFocus,
									  FW_Boolean isRoot)	// Override
{
	if (hasMenuFocus)
	{
		//---- Set up the Edit menu items ----
#ifdef FW_BUILD_MAC
		menuBar->EnableCommand(ev, kODCommandPaste, HasPropertyOnClipboard(ev, kODPropContents, FW_CPart::gMacTEXTDataType));
#endif
	}

	return FALSE;
}

//----------------------------------------------------------------------------------------
// CFormFrame::Draw
//----------------------------------------------------------------------------------------

void CFormFrame::Draw(Environment *ev, ODFacet* odFacet, ODShape* invalidShape)	// Override
{
	FW_CViewContext		gc (ev, this, odFacet, invalidShape);

	// fill with background color
	FW_CInk		fillInk (FW_CColor (221, 221, 221));	// LightGray
	FW_CRect	bounds = this->GetBounds(ev);	
	FW_CRectShape::RenderRect (gc, bounds, FW_kFill, fillInk);
}

//----------------------------------------------------------------------------------------
// CFormFrame::FrameShapeChanged
//----------------------------------------------------------------------------------------
// By default a FW_CFrame view is not refreshed entirely when resized.
// You must decide if your content's appearance depends on the frame's size.
// If it doesn't you don't need to override FrameShapeChanged().
// Here we must redraw the whole frame everytime because the text is centered.

void CFormFrame::FrameShapeChanged(Environment* ev)
{
	FW_CFrame::FrameShapeChanged(ev);

	this->Invalidate(ev);
}

//----------------------------------------------------------------------------------------
//	CFormFrame::NewClipboardCommand
//----------------------------------------------------------------------------------------

FW_CClipboardCommand* CFormFrame::NewClipboardCommand(Environment* ev, ODCommandID commandID)
{
	return FW_NEW(CFormEditCommand, (ev, commandID, fFormContent, this));
}

//----------------------------------------------------------------------------------------
//	CFormFrame::NewDragCommand
//----------------------------------------------------------------------------------------

FW_CDragCommand* CFormFrame::NewDragCommand(Environment *ev, 
											FW_CFrame* theFrame,
											const FW_CMouseEvent& theMouseEvent)
{
	return FW_NEW(CFormDragCommand, (ev, fFormContent, theFrame));
}

//----------------------------------------------------------------------------------------
//	CFormFrame::NewDropCommand
//----------------------------------------------------------------------------------------

FW_CDropCommand* CFormFrame::NewDropCommand(Environment* ev,
											 FW_CFrame* frame,
											 ODDragItemIterator* dropInfo, 
											 ODFacet* odFacet, 
											 const FW_CPoint& dropPoint)
{
	return FW_NEW(CFormDropCommand, (ev, fFormContent, this, dropInfo, odFacet, dropPoint));
}

//----------------------------------------------------------------------------------------
//	CFormFrame::CreateSubViews
//----------------------------------------------------------------------------------------

void CFormFrame::CreateSubViews(Environment* ev)
{
	// WARNING: Make sure that classes created from resources won't be dead-stripped
	// 			Use the macro FW_DO_NOT_DEAD_STRIP for those classes which are not 
	// 			referenced anywhere else in your part's code.
	FW_DO_NOT_DEAD_STRIP(FW_CGrowBox);
	FW_DO_NOT_DEAD_STRIP(FW_CScrollBar);
	FW_DO_NOT_DEAD_STRIP(FW_CScrollBarScroller);
	FW_DO_NOT_DEAD_STRIP(CPictView);
	FW_DO_NOT_DEAD_STRIP(FW_CEditView);
	FW_DO_NOT_DEAD_STRIP(FW_CButton);
	FW_DO_NOT_DEAD_STRIP(CRadioGroup);
	FW_DO_NOT_DEAD_STRIP(FW_CGroupBox);
	FW_DO_NOT_DEAD_STRIP(FW_CButton);
	FW_DO_NOT_DEAD_STRIP(FW_CRadioCluster);
	FW_DO_NOT_DEAD_STRIP(FW_CListBox);
	FW_DO_NOT_DEAD_STRIP(FW_CButton);
	FW_DO_NOT_DEAD_STRIP(FW_CPopupMenu);
	FW_DO_NOT_DEAD_STRIP(FW_CGroupBox);

	CreateSubViewsFromResource(ev, kMainWindowResID);
}

//----------------------------------------------------------------------------------------
//	CFormFrame::HandleNotification
//----------------------------------------------------------------------------------------

void CFormFrame::HandleNotification(Environment* ev, const FW_CNotification& notification)
{
	// Handle notification messages for the registered controls.
	// GetMessage() allows to switch on the type of notification.  For each message
	// we can down-cast "notification" directly to the right subclass, without
	// having to use RTTI, because the message is unique to that type.

	switch (notification.GetMessage()) 
	{
		default:
			FW_ASSERT("CFormFrame can't respond to this");
			break;
	}
}
